function [threshv] = extrapthresh(thresh,cpd)
%EXTRAPTHRESH Extrapolates contrast thresholds given measured thresholds.
%              Input is a small vector of experimentally measured threshold
%              values and the output is a larger vector of values. Does
%              linear extrapolation in log2 space.
%                            THRESHV = EXTRAPTHRESH( THRESH )
%               THRESH contains 6 measured values for [0.5,1,2,4,8,16] cpd.
%               THRESHV is 14 extrapolated thresholds for [1/32 cpd - 256cpd],
%					 plus a second column, which is the spat. freq in cpd
%
% History:
%   o 5/29/96:  bjs - added plotting section
%   o 8/04/00:   jn - changed output!
%================================================================================

  DOPLOT = 0;

% Vector of desired Cycles Per Degree(cpd) (in log2)
  x2  = [-5:9];
  y2 = ones(1,14); %The desired size of the interpolated date


% for backwards compatibility...
  if(nargin==1),    % default option for the input `cpd`
    cpd = [0.5 1 2 4 8 16];
  end;  
%thresh = [.004,.003,.004,.004,.014,.033]
  x0  = log2(cpd); % [-1  0  1  2  3  4]
  y0  = log2(thresh); %[-7.9658 -8.3808 -7.9658 -7.9658 -6.1584 -4.9214]
  cmn = min(x0); % -1
  cmx = max(x0); % 4
  ind = find(x2>=cmn & x2<=cmx); %[5 6 7 8 9 10] i.e. x2(5), x2(6)...
  imn = min(ind); % 5
  imx = max(ind); % 10

%-----------------------------------------------------
% Interpolate measured results onto desired spacing...
%(all in the log2 domain)

  y2(ind) = interp1(x0,y0,x2(ind));%[1 1 1 1 -7.9 -8.3 -7.9 -7.9 -6.1 -4.9 1 1 1 1]


%-----------------------------------------------------
% Extrapolate upwards 
%according to y=mx+b, finding m and b according to the last two points

  if(imx<14),
    m = (y2(imx)-y2(imx-1))/(x2(imx)-x2(imx-1));%angle of line
    b = y2(imx-1) - m*x2(imx-1);
    y2(imx+1:14) = m*x2(imx+1:14)+b;
    ind2 = [imx+1:14];
  end;

%-----------------------------------------------------
% Extrapolate downwards
%according to y=mx+b, finding m and b according to the first two points

  if(imn>1),
    m=(y2(imn+1)-y2(imn))/(x2(imn+1)-x2(imn));
    b=y2(imn)-m*x2(imn);
    y2(1:imn-1) = m*x2(1:imn-1)+b;
    ind2 = [1:imn-1 ind2];
  end;

  threshv = 2.^y2;

% --- Optional plotting ---
  if(DOPLOT),
    fig=figure(1);
    clf;
    set(gcf,'Name','Thresholds','NumberTitle','off');
    plot(x2,y2,'-',x2(ind),y2(ind),'co',x2(ind2),y2(ind2),'r+'); grid;
    hold on;
    plot(x0,y0,'m*');
    hold off;
    axis equal
    xlabel('log2(cyc/deg)')
    ylabel('log2(threshold)');
    title('Linear Threshold Extrapolation');
    hl=legend('m*',' measured ','co',' interpolated ','r+',' extrapolated ');
    axes(hl); refresh;
    keyboard;
  end;

threshv=[threshv; 1/32 1/16 1/8 1/4 1/2 1 2 4 8 16 32 64 128 256];
threshv=threshv';
  return;

