% functions:
%extrapthresh, logcosine, hilbert2d
function [orig, imgecc, tbl, tb1_energy] = img_ecc2(img, X, eccentricity, x0, y0)
% Based on multOrient2 performs the visual based feature extraction 
% for different eccentricities - all eccentricities and orders
%-------------------------------------------------------------

displayBands = 1;            % display: 1 = bandpass, 0 = edges
trackAngle = -1;	            %0,90

Hcut = 0;            % How many high-order octaves to drop
Lcut = 1;            % How many low-order octaves to drop (Lcut-1)

num = 2;                      % how many orientations?
cscale = 0; fscale = 0;			% randomization parameters

fprintf('loading %s...\n',img);

orig  = X;
[R, C] = size(orig);
% Calculate number of orders from size;
Lowest = Lcut; Highest = round(log2(R/2)-Hcut);

fprintf('calculating threshold...\n');
% The extrapolated threshold vector (for orders 3 - 8)
%CSF_fovea=[.004,.003,.004,.004,.014,.033];
thM = extrapthresh([.004,.003,.004,.004,.014,.033]);%extrapolate up and down

Img = fft2(orig);         
dc = Img(1,1)/(R*C); % the (0,0) in Fourier is the average*R*C
Img = fftshift(Img); %and therefore dc is the average of the real spatial image
%dc is to normalize the contrast and the image
%------------------------------------------------------------
%The threshold model: threshold=exp{a*eccentricity*frequency+ln(foveal_thresh)}
a=0.04; %a constant in the model
deg=4; %FOV of the image
min_lg_f=round(log2(1/deg))+2;%min log frequency in cyc/deg (cycles per 1 degree)
max_lg_f=round(log2((R/2)/deg))+2;%max log frequency in cyc/deg 
%d_lg_f=(max_lg_f-min_lg_f);
lg_cpd=[min_lg_f:1:max_lg_f];%log frequency coordinates in cyc/deg (needed in the model)
cpd=2.^lg_cpd;  %frequency coordinates in cyc/deg
ex_fov_CSF=thM(:,1);
%thrf1=find(thM(:,2)==1/deg)+1;
thrf1=find(thM(:,2)==1/deg);%first thresh frequency index
thrf2=find(thM(:,2)==round((R/2)/deg));%last thresh frequency index
%thrf2=12
ex_fov_CSF = ex_fov_CSF(thrf1:thrf2);ex_fov_CSF=ex_fov_CSF';%extrapolated foveal CSF

%%%%%%%%%%%%% NEW %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%eccentricity=0:4:36;
max_ecc=32;
%eccentricity=0:max_ecc/(R/sqrt(2)):max_ecc;
eccentricity=0:max_ecc/R:max_ecc;
Num_rings=length(eccentricity);
Num_orders=length(ex_fov_CSF);
CSTresh=zeros(Num_rings,Num_orders);

for ring=1:1:Num_rings
ln_CSF_ecc=a*eccentricity(ring)*2.^lg_cpd+log2(ex_fov_CSF); %or log
CSF_ecc=2.^ln_CSF_ecc;
CSTresh(ring,:)=CSF_ecc;
end

figure;
plot(CSTresh);
legend('order 1','order 2','order 3','order 4','order 5','order 6','order 7','order 8');
xlabel('Eccentricity [pixels]');
ylabel('CST');
axis([1 150 0 1]);
figure;
semilogy(CSTresh');
xlabel('Order');
ylabel('CST');

%x0 and y0 are the fovea location in the image
CST_2D=zeros(R+1,R+1,8);
R1=R/2;
fovea_locX=x0;fovea_locY=y0;
%fovea_locX=110;fovea_locY=80;
%fovea_locX=222;fovea_locY=142;
%fovea_locX=R/2;fovea_locY=R/2;
for order = 1:1:8
for i=1:1:R
   for j=1:1:R
      ind=round(sqrt((i-fovea_locX).^2 + (j-fovea_locY).^2));
      CST_2D(i,j,order)=CSTresh(ind+1,order);
   end
end
end
CST_2D=CST_2D(1:R,1:R,:);

figure;colormap(gray);
imagesc(CST_2D(:,:,1));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%-----------------------------------------------------------
imgecc=zeros(R);
total_th_bpimg=zeros(R);
fprintf('calculating orders...\n');
for this_order = Lowest:Highest
    fprintf('%d ', this_order);
    eval(sprintf('bpimg%d = logcosine(Img,%d);', ...
        this_order, this_order));
    for j = 0:num-1
        this_angle = round(180/num*j) ;
        eval(sprintf('img%d_%d = real(hilbert2d(bpimg%d,%d,num));', ... 
            this_order, this_angle, this_order, this_angle));
    end
end

fprintf('\nlocating congruent values...\n');

% initialize
pos = ones(R,C); neg = ones(R,C);

for j = 0:num-1
    this_angle = round(180/num*j) ;
    eval(sprintf('pos%d = ones(R,C);', this_angle));
    eval(sprintf('neg%d = ones(R,C);', this_angle));
end	

ec = (eccentricity/4)+2; %arranging the table of energy
ec=round(ec);
if (eccentricity == 0)
    ec = 1;
elseif (eccentricity == 2)
    ec = 2;
end

for this_order = Lowest:Highest
    
    eval(sprintf('bpimgall = bpimg%d ;', this_order));
   
    if (eccentricity ==0)
        tbl(1,this_order) = (sum(sum( abs( bpimgall))) )/(256*256);
        tb1_energy(1,this_order) = (sum(sum( (abs( bpimgall)).^2)) )/(256*256);
    end
    
    %thr = thM(this_order + tbase);% 6,7,8,9
    thr = CSF_ecc(this_order+1);
    
    fprintf('Order: %i Threshold used:%g,  \n', this_order, thr);
    
    % HIGH AND LOW: values that are true positive or
    %               negative (ie, not within threshold)
    high = (bpimgall > thr*dc); %Each scale can be thresholded in the space domain because we assume a single frequency?
    low = (bpimgall < -thr*dc);     
    
%%%%%%%%%%%%%%%% NEW %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

%eval(sprintf('subplot(3,3,%d);',this_order));
    figure(22);colormap(gray);
    if this_order<10 eval(sprintf('subplot(3,3,%d);',this_order)); 
    imagesc(bpimgall); end
bpimgall=bpimgall/max(max(abs(bpimgall)));
    eval(sprintf('th_bpimg%d = zeros (R,R);',this_order));
    eval(sprintf('above_thr=find(abs(bpimgall) > (CST_2D(:,:,%d)));',this_order));
    eval(sprintf('th_bpimg%d(above_thr)= bpimgall(above_thr);',this_order));

    %eval(sprintf('th_bpimg%d = bpimg%d - CST_2D(:,:,%d);',this_order,this_order,this_order));
    eval(sprintf('total_th_bpimg = total_th_bpimg + th_bpimg%d;',this_order));
  if this_order<10 
    figure(23);colormap(gray);
        eval(sprintf('subplot(3,3,%d);',this_order)); 
        eval(sprintf('imagesc(th_bpimg%d);',this_order));     
    figure(24);colormap(gray);
        eval(sprintf('subplot(3,3,%d);',this_order)); 
        imagesc(total_th_bpimg); 
   end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    
    %    eval(sprintf('total_int(ec,this_order)=(sum(abs(bpimg%d(high)))+sum(abs(bpimg%d(low))))/(dc);',this_order,this_order))%%%%%has been added
    %    total_pix(ec,this_order)=(sum(sum(abs(high)))+sum(sum(abs(low))))/(R*C)%%%%%   has been added
    %    
    %    total_energy(ec,this_order) = sum(sum(abs(low .* bpimgall)/(256*256/16.544))) 
    tbl(ec+1,this_order) = (sum(sum( abs(low .* bpimgall))) + sum(sum(abs(high .* bpimgall) )))/(256*256);
    tb1_energy(ec+1,this_order) = (sum(sum( (abs(low .* bpimgall)).^2)) + sum(sum( (abs(high .* bpimgall) ).^2)))/(256*256) ;
    
    eval(sprintf('imgecc(high)=imgecc(high)+bpimg%d(high);',this_order));
    eval(sprintf('imgecc(low)=imgecc(low)+bpimg%d(low);',this_order));
    
    % POS AND NEG: cumulative image from ANDing orders
    
    pos = pos & high;
    neg = neg & low;
    
    % HIGH_ORDER_ANGLE AND LOW_ORDER_ANGLE: oriented orders
    % POS_ANGLE AND NEG_ANGLE: cumulative oriented
    
    for j = 0:num-1
        this_angle = round(180/num*j) ;      
        eval(sprintf('high%d_%d = (img%d_%d > thr*dc);', ...
            this_order, this_angle, this_order, this_angle));   
        eval(sprintf('pos%d = pos%d & high%d_%d;', ...
            this_angle, this_angle, this_order, this_angle));  
        
        eval(sprintf('low%d_%d = (img%d_%d < -thr*dc);', ...
            this_order, this_angle, this_order, this_angle));  
        eval(sprintf('neg%d = neg%d & low%d_%d;', ...
            this_angle, this_angle, this_order, this_angle));  
        
        
    end % angle
    
    % HIGH_ORDER AND LOW_ORDER are whatever we're printing
    %         (assuming we print bands)
    
    
    if (trackAngle >= 0)
        eval(sprintf('high%d = high%d_%d;', ...
            this_order , this_order, trackAngle));
        eval(sprintf('low%d = low%d_%d;', ...
            this_order , this_order, trackAngle));
    else 
        eval(sprintf('high%d = high; low%d = low;', ...
            this_order, this_order));
    end
    
end % order
colormap(gray);
imgecc=total_th_bpimg;
