function [orig, orig1, ind4, ind6] = multOrient2(img, X)
%MULTORIENT  Extracts edges based on congruence of
%	          phase, filtering non-Hilbert with multiple directions.
%            Based on orien2Extract
%
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% file: multOrient.m
% author: Jack Nye, Schepens Eye Research Inst.
% date: 5 May, 1998
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Image registration from bi-polar edges (Fig. 1) compared to registration 
% from the shifted images. SAVD of an image subsection

%dir = 'd:\images\bw';  % directory with images
%load clena256
%img = 'clena256';
%load trees
%rect=[1 1 255 255]; X=imcrop(X,rect);
%img='X';
displayBands = 1;            % display: 1 = bandpass, 0 = edges
trackAngle = -1;	%0,90			  % Display which orientation?

Hcut = 1;            % How many high-order octaves to drop
Lcut = 3;            % How many low-order octaves to drop (Lcut-1)

num = 2;                      % how many orientations?
cscale = 0; fscale = 0;			% randomization parameters

fprintf('loading %s...\n',img);

%eval(sprintf('load %s\\%s;', dir, img));
eval(sprintf('orig =  %s;', img));

[R, C] = size(orig);

% Calculate number of orders from size;
Lowest = Lcut; Highest = round(log2(R/2)-Hcut); %@

fprintf('calculating threshold...\n');
  % The extrapolated threshold vector (for orders 3 - 8)
thM = extrapthresh1([.004,.003,.004,.004,.014,.033 1]);
%thM = extrapthresh1([.004,.003,.004,.004,.014,.1 1]);

tbase = 3;  %@This is based on a 4 degree image,
% where we start at 1/4 c/deg. <tbase> is the offset in the 
% extrapolated threshold vector based on image size in degrees.
%thM is the extrapolated thresh. between 2^-5 to 2^8,
%1/4 is 2^-2 so we start from (2^-5+3) = 2^-5+tbase

Img = fft2(orig);         
dc = Img(1,1)/(R*C); % the (0,0) in Fourier is the average*R*C
Img = fftshift(Img); %and therefore dc is the average of the real spatial image
                     %dc is to normalize the contrast and the image
fprintf('calculating orders...\n');
for this_order = Lowest:Highest
   fprintf('%d ', this_order);
   eval(sprintf('bpimg%d = logcosine(Img,%d);', ...
      this_order, this_order));
   for j = 0:num-1
      this_angle = round(180/num*j) ;
      eval(sprintf('img%d_%d = real(hilbert2d(bpimg%d,%d,num));', ... 
         this_order, this_angle, this_order, this_angle));
   end
end


fprintf('\nlocating congruent values...\n');

% initialize
pos = ones(R,C); neg = ones(R,C);

for j = 0:num-1
   this_angle = round(180/num*j) ;
   eval(sprintf('pos%d = ones(R,C);', this_angle));
   eval(sprintf('neg%d = ones(R,C);', this_angle));
end	

for this_order = Lowest:Highest
   
   thr = thM(this_order + tbase);% 6,7,8,9
   
   fprintf('Order: %i Threshold used:%g,  \n', this_order, thr);
   
   % HIGH AND LOW: values that are true positive or
   %               negative (ie, not within threshold)
   eval(sprintf('high = (bpimg%d > thr*dc);', this_order));
   eval(sprintf('low = (bpimg%d < -thr*dc);', this_order));     
   %total_ord=(sum(sum(abs(high)))+sum(sum(abs(low))))/(R*C)
   % POS AND NEG: cumulative image from ANDing orders
   
   pos = pos & high;
   neg = neg & low;
   
   % HIGH_ORDER_ANGLE AND LOW_ORDER_ANGLE: oriented orders
   % POS_ANGLE AND NEG_ANGLE: cumulative oriented
   
   for j = 0:num-1
      this_angle = round(180/num*j) ;      
      eval(sprintf('high%d_%d = (img%d_%d > thr*dc);', ...
         this_order, this_angle, this_order, this_angle));   
      eval(sprintf('pos%d = pos%d & high%d_%d;', ...
         this_angle, this_angle, this_order, this_angle));  
      
      eval(sprintf('low%d_%d = (img%d_%d < -thr*dc);', ...
         this_order, this_angle, this_order, this_angle));  
      eval(sprintf('neg%d = neg%d & low%d_%d;', ...
         this_angle, this_angle, this_order, this_angle));  
      
      
   end % angle
   
   % HIGH_ORDER AND LOW_ORDER are whatever we're printing
   %         (assuming we print bands)
   
   
   if (trackAngle >= 0)
      eval(sprintf('high%d = high%d_%d;', ...
         this_order , this_order, trackAngle));
      eval(sprintf('low%d = low%d_%d;', ...
         this_order , this_order, trackAngle));
   else 
      eval(sprintf('high%d = high; low%d = low;', ...
         this_order, this_order));
   end
   
end % order

 % FOUND EDGES
   
   orig1 = ones(R,C) *128;
   high=ones(R,C); low=ones(R,C);
   
   for this_order = Lowest:Highest
      eval(sprintf('high = high & high%d;', this_order));
      eval(sprintf('low = low & low%d;', this_order));
   end
   
   ind4 = find(high);
   ind6 = find(low);
   
   % high -> white    low->black
   orig1(ind4)=256*ones(size(ind4));
   orig1(ind6)=1*ones(size(ind6));
   
   return;

