function [F] = radialcos2(R,C,center,width)
% RADIALCOS2  Computes a cosine^2 filter in radial space.
%                  F = RADIALCOS2(R,C,CENTER,WIDTH)
%                       F = filter
%                     R,C = image size (Rows x Columns)
%                  CENTER = center angle
%                   WIDTH = angular filter width to 0.5 level.
%
%             See also: LOGCOSINE

%  Written by B. Sperry
%  6/16/95   bjs - replaced rmtrx, cmtrx fxn calls with quicker
%                  "tony" option - 6x faster for 256x256 images!!
%=====================================================================

  if(nargin ~= 4), error('[F] = RADIALCOS2(R,C,CENTER,WIDTH)'); end;
  
  center = rem(center,180);
  width  = rem(width,360);
  
  c_rad = center/180*pi;
  w_rad = width/180*pi;

% --- Get dc point (works for R,C even or odd) ---
  Xdc = fix(C/2) + 1;
  Ydc = fix(R/2) + 1;

% --- Create coord sys with dc exactly centered ---
  c = 2*Xdc - 1;
  r = 2*Ydc - 1;
  x = 1:c; y=[1:r]';
  X = x(ones(1,r),:) - Xdc;
  Y = y(:,ones(1,c)) - Ydc;

  theta = -atan2(Y,X);		  % '-' is so that +'ve theta is CCW from east
  alpha = flipud(fliplr(theta));

  F0 = zeros(r,c);
  phi = theta - c_rad;
  F0(abs(phi)<=w_rad) = cos( ( phi(abs(phi)<=w_rad) -w_rad ) /w_rad*pi/2 +pi/2 ).^2;
  phi = alpha - c_rad;
  F0(abs(phi)<=w_rad) = cos((phi(abs(phi)<=w_rad)-w_rad)/w_rad*pi/2 + pi/2).^2;

% --- Select out original coord sys from working one ---
%     Note that if R,C both odd, the two systems are identical.
  F = F0(1:R,1:C);


  return;
